package com.tanpu.community.feign.fatools;

import com.tanpu.common.api.CommonResp;
import com.tanpu.community.api.beans.qo.feign.fatools.UserInfoCorpReq;
import com.tanpu.community.api.beans.req.page.Page;
import com.tanpu.community.api.beans.resp.SysConstantResp;
import com.tanpu.community.api.beans.vo.feign.course.ShortVideoBaseInfoResp;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoChiefInvAdvResp;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoCorpResp;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoNewChief;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoOrg;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoResp;
import com.tanpu.community.api.beans.vo.feign.user.FileQueryResp;
import com.tanpu.community.api.beans.vo.feign.user.UserInfoVo;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@Component
public class FeignbackForFatools implements FallbackFactory<FeignClientForFatools> {

    @Override
    public FeignClientForFatools create(Throwable throwable) {
        return new FeignClientForFatools() {
            @Override
            public UserInfoVo getUserInfoById(String userId) {
                log.info("FeignbackForFatools.getUserInfoById调用失败", throwable);
                return null;
            }

            @Override
            public List<ShortVideoBaseInfoResp> getCurriculumByColumnRelId(List<String> ids) {
                log.info("FeignbackForFatools.getCurriculumByColumnRelId调用失败", throwable);
                return null;
            }

            @Override
            public Map<String, FileQueryResp> queryFileMap(List<String> ids) {
                log.info("FeignbackForFatools.getCurriculumByColumnRelId调用失败", throwable);
                return null;
            }

            @Override
            public CommonResp<Set<String>> getNotRelaySet(String userId, Set roomIds) {
                log.info("FeignbackForFatools.getNotRelaySet", throwable);
                return CommonResp.error();
            }

            @Override
            public CommonResp<UserInfoResp> queryUserInfoNew(String userId) {
                log.info("FeignbackForFatools.queryUsersListNew", throwable);
                return CommonResp.error();
            }

            @Override
            public List<UserInfoResp> queryUserListNew(List<String> userIds) {
                log.info("FeignbackForFatools.queryUserListNew", throwable);
                return Collections.emptyList();
            }

            @Override
            public CommonResp<List<UserInfoOrg>> getUsetInfoByOrgUserId(String orgUserId) {
                log.info("FeignbackForFatools.getUsetInfoByOrgUserId", throwable);
                return CommonResp.error();
            }

            @Override
            public CommonResp<Page<UserInfoNewChief>> queryChiefFinancialAdviserList(Integer pageNumber, Integer pageSize) {
                log.info("FeignbackForFatools.queryChiefFinancialAdviserList", throwable);
                return CommonResp.error();
            }

            @Override
            public List<UserInfoResp> queryByUserNameMp(String keyword, Integer pageSize, Integer pageNumber) {
                log.info("FeignbackForFatools.queryByUserNameMp", throwable);
                return null;
            }

            @Override
            public CommonResp<Page<UserInfoCorpResp>> queryCorpListWithExcludeId(UserInfoCorpReq req) {
                log.error("FeignbackForFatools.queryByUserNameMp", throwable);
                return CommonResp.error();
            }

            @Override
            public CommonResp<Page<UserInfoChiefInvAdvResp>> queryChiefInvAdvListWithExcludeId(UserInfoCorpReq req) {
                log.error("FeignbackForFatools.queryByUserNameMp", throwable);
                return CommonResp.error();
            }

            @Override
            public CommonResp<List<SysConstantResp>> queryLabels(String constantGroup) {
                log.error("FeignbackForFatools.queryLabels", throwable);
                return CommonResp.error();
            }
        };
    }

}
